# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
 
 
setup(
    name='vcopt',
    version='1.5.3',
    description='STOP thinking, just use Viniette&Clarity Genetic Algorithm optimizer.',
    long_description='STOP thinking, just use Viniette&Clarity Genetic Algorithm optimizer.',
    author='Shoya Yasuda @ Viniette&Clarity, Inc.',
    author_email='selamatpagi1124@gmail.com',
    url='https://github.com/',
    license='MIT License',
    packages=find_packages(),
    install_requires=['numpy', 'matplotlib'],
)